package co.ab180.airbridge.flutter

import android.app.Application
import android.content.Intent
import co.ab180.airbridge.Airbridge
import co.ab180.airbridge.AirbridgeConfig
import co.ab180.airbridge.flutter.utility.setFromAirbridgeJSON
import io.flutter.embedding.engine.plugins.FlutterPlugin

class AirbridgeFlutter: FlutterPlugin {
    companion object {
        @JvmStatic
        fun init(application: Application, appName: String, appToken: String) {
            Airbridge.init(
                app = application,
                config = AirbridgeConfig
                    .Builder(appName, appToken)
                    .setFromAirbridgeJSON(application)
                    .setPlatform("flutter")
                    .build()
            )
        }

        @JvmStatic
        fun processDeeplink(intent: Intent) {
            DeeplinkAPI.deeplinkTracker.provideIntent(intent)
        }
    }

    private lateinit var deeplinkAPI: DeeplinkAPI
    private lateinit var eventAPI: EventAPI
    private lateinit var stateAPI: StateAPI
    private lateinit var placementAPI: PlacementAPI

    override fun onAttachedToEngine(binding: FlutterPlugin.FlutterPluginBinding) {
        deeplinkAPI = DeeplinkAPI.fromPlugin(binding).apply { attachToChannel() }
        eventAPI = EventAPI.fromPlugin(binding).apply { attachToChannel() }
        stateAPI = StateAPI.fromPlugin(binding).apply { attachToChannel() }
        placementAPI = PlacementAPI.fromPlugin(binding).apply { attachToChannel() }
    }

    override fun onDetachedFromEngine(binding: FlutterPlugin.FlutterPluginBinding) {
        deeplinkAPI.detachFromChannel()
        eventAPI.detachFromChannel()
        stateAPI.detachFromChannel()
        placementAPI.detachFromChannel()
    }
}