package co.ab180.airbridge.flutter

import co.ab180.airbridge.Airbridge
import io.flutter.embedding.engine.plugins.FlutterPlugin
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel

class PlacementAPI(private val channel: MethodChannel) : MethodChannel.MethodCallHandler {
    companion object

    fun attachToChannel() {
        channel.setMethodCallHandler(this)
    }

    fun detachFromChannel() {
        channel.setMethodCallHandler(null)
    }

    override fun onMethodCall(call: MethodCall, result: MethodChannel.Result) {
        val arguments = call.arguments as? Map<*, *> ?: HashMap<Any, Any>()

        when (call.method) {
            "click" -> click(arguments, result)
            "impression" -> impression(arguments, result)
            else -> result.notImplemented()
        }
    }

    private fun click(arguments: Map<*, *>, result: MethodChannel.Result) {
        (arguments["trackingLink"] as? String)?.let { trackingLink ->
            Airbridge.click(
                trackingLink = trackingLink
            )
        }

        result.success(null)
    }

    private fun impression(arguments: Map<*, *>, result: MethodChannel.Result) {
        (arguments["trackingLink"] as? String)?.let { trackingLink ->
            Airbridge.impression(
                trackingLink = trackingLink
            )
        }

        result.success(null)
    }
}

fun PlacementAPI.Companion.fromPlugin(binding: FlutterPlugin.FlutterPluginBinding): PlacementAPI {
    return PlacementAPI(MethodChannel(binding.binaryMessenger, "airbridge_flutter_sdk/method/placement"))
}