package co.ab180.airbridge.flutter.utility

import co.ab180.airbridge.Airbridge
import co.ab180.airbridge.AirbridgeUser

fun AirbridgeUser.set(user: Map<*, *>?) {
    if (user == null) {
        Airbridge.getCurrentUser().id = null
        Airbridge.getCurrentUser().email = null
        Airbridge.getCurrentUser().phone = null
        Airbridge.getCurrentUser().clearAlias()
        Airbridge.getCurrentUser().clearAttributes()
    }

    user?.run {
        (get("id") as? String).let { Airbridge.getCurrentUser().id = it }
        (get("email") as? String).let { Airbridge.getCurrentUser().email = it }
        (get("phone") as? String).let { Airbridge.getCurrentUser().phone = it }
        (get("alias") as? Map<*, *>).let { Airbridge.getCurrentUser().setAlias(it) }
        (get("attributes") as? Map<*, *>).let { Airbridge.getCurrentUser().setAttribute(it) }
    }
}

fun AirbridgeUser.update(user: Map<*, *>?) {
    user?.run {
        (get("id") as? String)?.let { Airbridge.getCurrentUser().id = it }
        (get("email") as? String)?.let { Airbridge.getCurrentUser().email = it }
        (get("phone") as? String)?.let { Airbridge.getCurrentUser().phone = it }
        (get("alias") as? Map<*, *>)?.let { Airbridge.getCurrentUser().setAlias(it) }
        (get("attributes") as? Map<*, *>)?.let { Airbridge.getCurrentUser().setAttribute(it) }
    }
}

fun AirbridgeUser.setAlias(map: Map<*, *>?) {
    Airbridge.getCurrentUser().clearAlias()
    map?.apply {
        entries.forEach {
            (it.key as? String)?.let { key ->
                (it.value as? String)?.let { value ->
                    Airbridge.getCurrentUser().setAlias(key, value)
                }
            }
        }
    }
}

fun AirbridgeUser.setAttribute(map: Map<*, *>?) {
    Airbridge.getCurrentUser().clearAttributes()
    map?.apply {
        entries.forEach {
            (it.key as? String)?.let { key ->
                (it.value as? String)?.let { value ->
                    Airbridge.getCurrentUser().setAttribute(key, value)
                }
                (it.value as? Int)?.let { value ->
                    Airbridge.getCurrentUser().setAttribute(key, value)
                }
                (it.value as? Long)?.let { value ->
                    Airbridge.getCurrentUser().setAttribute(key, value)
                }
                (it.value as? Float)?.let { value ->
                    Airbridge.getCurrentUser().setAttribute(key, value)
                }
                (it.value as? Double)?.let { value ->
                    Airbridge.getCurrentUser().setAttribute(key, value.toFloat())
                }
                (it.value as? Boolean)?.let { value ->
                    Airbridge.getCurrentUser().setAttribute(key, value)
                }
            }
        }
    }
}