import 'package:airbridge_flutter_sdk_example/basic/component/button.dart';

import 'package:flutter/material.dart';

class ButtonData {
  final String text;
  final void Function() onPressed;

  ButtonData(this.text, this.onPressed);
}

class ButtonGrid extends StatelessWidget {
  final int width;
  final Color backColor;
  final Color textColor;
  final List<ButtonData> datas;

  ButtonGrid({
    required this.width,
    required this.backColor,
    required this.textColor,
    required this.datas,
  });

  Column getColumn() {
    int length = (datas.length / width).ceil();

    List<Widget> children = [];
    for (int i = 0; i < length; i++) {
      children.add(getRow(i));
    }

    return Column(
      children: children,
    );
  }

  Row getRow(int index) {
    int startPoint = index * width;
    int length = datas.length - startPoint;
    length = length > width ? width : length;

    double halfBlank = (width - length) / 2;
    int leftBlank = halfBlank.floor();
    int rightBlank = halfBlank.floor();
    if (halfBlank.floor() == halfBlank - 0.5) {
      rightBlank += 1;
    }

    List<Widget> children = [];
    for (int i = 0; i < leftBlank; i++) {
      children.add(
          Expanded(
            child: Center(),
          )
      );
    }
    for (int i = startPoint; i < startPoint+length; i++) {
      ButtonData data = datas.elementAt(i);

      children.add(
          Expanded(
              child: Button(
                data.text,
                backColor: backColor,
                textColor: textColor,
                onPressed: data.onPressed,
              )
          )
      );
    }
    for (int i = 0; i < rightBlank; i++) {
      children.add(
          Expanded(
            child: Center(),
          )
      );
    }

    return Row(
        children: children
    );
  }

  @override
  Widget build(BuildContext context) {
    return Container(
        child: getColumn()
    );
  }
}