import 'package:airbridge_flutter_sdk_example/basic/component/status_row.dart';
import 'package:airbridge_flutter_sdk_example/basic/utility/storage.dart';

import 'package:flutter/material.dart';
import 'package:intl/intl.dart';

import 'data/log_item.dart';

class LogPage extends StatefulWidget {
  @override
  State<StatefulWidget> createState() {
    return _State();
  }
}

class _State extends State<LogPage> {
  List<LogItem> items = [];

  late Function logCallback;

  _State() {
    logCallback = (logs) {
      items = logs;
      setState(() {});
    };
  }

  @override
  void initState() {
    super.initState();

    items = Storage.get('logs');
    setState(() {});

    Storage.addOnSet('logs', logCallback);
  }

  @override
  void dispose() {
    super.dispose();
    Storage.removeOnSet('logs', logCallback);
  }

  @override
  Widget build(BuildContext context) {
    return ListView.builder(
        padding: EdgeInsets.all(6.0),
        itemCount: items.length,
        itemBuilder: (BuildContext context, int index) {
          return _StatusRow(
            name: DateFormat('hh:mm a').format(items[index].date),
            status: items[index].message,
            onPressed: () {
              show('log', items[index].message);
            },
          );
        }
    );
  }

  void show(String title, String content) {
    Future.delayed(
        Duration.zero, () {
          showDialog(
            context: context,
            builder: (context) {
              return AlertDialog(
                title: Text(title),
                content: Text(content),
              );
            },
          );
        }
    );
  }
}

class _StatusRow extends StatelessWidget {
  final String name;
  final String status;
  final void Function() onPressed;

  _StatusRow({
    required this.name,
    required this.status,
    required this.onPressed
  });

  @override
  Widget build(BuildContext context) {
    return Card(
      margin: EdgeInsets.all(3.0),
      child: Container(
        padding: EdgeInsets.all(3.0),
        child: StatusRow(
          name: name,
          status: status,
          backColor: Colors.deepPurpleAccent,
          textColor: Colors.white,
          textAlign: TextAlign.left,
          onPressed: onPressed,
        ),
      ),
    );
  }
}