import Flutter
import UIKit
import AirBridge

@objc(AirbridgeFlutter_Swift)
public class AirbridgeFlutter: NSObject, FlutterPlugin {
    var deeplinkAPI: DeeplinkAPI!
    var eventAPI: EventAPI!
    var stateAPI: StateAPI!
    var placementAPI: PlacementAPI!
    
    public static func register(with registrar: FlutterPluginRegistrar) {
        let airbridge = AirbridgeFlutter()
        
        airbridge.attachToEngine(for: registrar)
    }
    
    func attachToEngine(for registrar: FlutterPluginRegistrar) {
        deeplinkAPI = DeeplinkAPI.fromPlugin(registrar)
        eventAPI = EventAPI.fromPlugin(registrar)
        stateAPI = StateAPI.fromPlugin(registrar)
        placementAPI = PlacementAPI.fromPlugin(registrar)
        
        deeplinkAPI.attachToChannel()
        eventAPI.attachToChannel()
        stateAPI.attachToChannel()
        placementAPI.attachToChannel()
    }
    
    public func detachFromEngine(for registrar: FlutterPluginRegistrar) {
        deeplinkAPI.detachFromChannel()
        eventAPI.detachFromChannel()
        stateAPI.detachFromChannel()
        placementAPI.detachFromChannel()
    }
}

extension AirbridgeFlutter {
    @objc
    public static func initSDK(
        appName: String, appToken: String,
        withLaunchOptions launchOptions: [AnyHashable: Any]?
    ) {
        AirBridge.setConfigFromAirbridgeJSON()
        AirBridge.state().setSDKDevelopmentPlatform("flutter")
        
        AirBridge.getInstance(appToken, appName: appName, withLaunchOptions: launchOptions)
        DeeplinkAPI.setInitialDeeplinkCallback()
    }
    
    @objc
    public static var deeplink: ABDeeplink {
        get {
            AirBridge.deeplink()
        }
    }
}
