//
//  DeeplinkAPI.swift
//  airbridge_flutter_sdk
//
//  Created by WOF on 2021/07/05.
//

import Foundation
import AirBridge

class DeeplinkAPI: NSObject, FlutterStreamHandler {
    let channel: FlutterEventChannel
    var deeplinkListenerID: UUID? = nil
    
    init(channel: FlutterEventChannel) {
        self.channel = channel
    }
}

extension DeeplinkAPI {
    func attachToChannel() {
        channel.setStreamHandler(self)
    }
    
    func detachFromChannel() {
        channel.setStreamHandler(nil)
    }
}

extension DeeplinkAPI {
    private static var deeplinkTracker = DeeplinkTracker()
    
    static func setInitialDeeplinkCallback() {
        AirBridge.deeplink().setDeeplinkCallback { url in
            DeeplinkAPI.deeplinkTracker.provideDeeplink(url)
        }
    }
    
    func onListen(withArguments arguments: Any?, eventSink events: @escaping FlutterEventSink) -> FlutterError? {
        let target = arguments as? String
        
        if target == "deeplink" {
            deeplinkListenerID = DeeplinkAPI.deeplinkTracker.listenDeeplink { deeplink in
                events(deeplink)
            }
        }
        
        return nil
    }
    
    func onCancel(withArguments arguments: Any?) -> FlutterError? {
        if let deeplinkListenerID = deeplinkListenerID {
            DeeplinkAPI.deeplinkTracker.stopListenDeeplink(id: deeplinkListenerID)
        }
        
        return nil
    }
}

fileprivate struct DeeplinkTracker {
    var startDeeplink: String? = nil
    private var deeplinkListeners: Dictionary<UUID, (String) -> Void> = [:]
    
    mutating func provideDeeplink(_ url: String) {
        trackDeeplink(url)
    }
    
    private mutating func trackDeeplink(_ url: String) {
        if startDeeplink == nil {
            startDeeplink = url
        }
        
        deeplinkListeners.values.forEach { $0(url) }
    }
    
    @discardableResult
    mutating func listenDeeplink(listener: @escaping (String) -> Void) -> UUID {
        if deeplinkListeners.count == 0 {
            if let startDeeplink = startDeeplink {
                listener(startDeeplink)
            }
        }
        
        let id = UUID()
        deeplinkListeners[id] = listener
        
        return id
    }
    
    mutating func stopListenDeeplink(id: UUID) {
        deeplinkListeners.removeValue(forKey: id)
    }
}

extension DeeplinkAPI {
    static func fromPlugin(_ registrar: FlutterPluginRegistrar) -> DeeplinkAPI {
        return DeeplinkAPI(channel: FlutterEventChannel(name: "airbridge_flutter_sdk/event/deeplink",
                                                        binaryMessenger: registrar.messenger()))
    }
}
