//
//  EventAPI.swift
//  airbridge_flutter_sdk
//
//  Created by WOF on 2021/07/05.
//

import Foundation
import AirBridge

struct EventAPI {
    let channel: FlutterMethodChannel
}

extension EventAPI {
    mutating func attachToChannel() {
        channel.setMethodCallHandler(handle(call:result:))
    }
    
    mutating func detachFromChannel() {
        channel.setMethodCallHandler(nil)
    }
}

extension EventAPI {
    func handle(call: FlutterMethodCall, result: FlutterResult) {
        let arguments = call.arguments as? [AnyHashable: Any] ?? [:]
        
        switch call.method {
        case "trackEvent": trackEvent(arguments, result)
        default: result(FlutterMethodNotImplemented)
        }
    }
}

extension EventAPI {

    func trackEvent(_ arguments: [AnyHashable: Any], _ result: FlutterResult) {
        guard let category = arguments["category"] as? String else {
            result(FlutterError.init(code: "BAD_ARGS",
                                     message: "category not found",
                                     details: nil))
            return
        }
        
        AirBridge.instance().trackEvent(category: category,
                                        option: arguments["option"] as? [AnyHashable: Any])
        
        result(nil)
    }
}

extension EventAPI {
    static func fromPlugin(_ registrar: FlutterPluginRegistrar) -> EventAPI {
        return EventAPI(channel: FlutterMethodChannel(name: "airbridge_flutter_sdk/method/event",
                                                      binaryMessenger: registrar.messenger()))
    }
}
