//
//  ABState.swift
//  airbridge_flutter_sdk
//
//  Created by WOF on 2021/07/05.
//

import Foundation
import AirBridge

extension ABState {
    func setUser(from dictionary: [AnyHashable: Any]?) {
        let user = ABUser()
        
        user.id = dictionary?["id"] as? String
        user.email = dictionary?["email"] as? String
        user.phone = dictionary?["phone"] as? String
        user.alias = (dictionary?["alias"] as? Dictionary)?.toAliasDictionary()
        user.attributes = (dictionary?["attributes"] as? Dictionary)?.toAttributesDictionary()
        
        setUser(user)
    }
    
    func updateUser(from dictionary: [AnyHashable: Any]?) {
        if let id = dictionary?["id"] as? String {
            setUserID(id)
        }
        if let email = dictionary?["email"] as? String {
            setUserEmail(email)
        }
        if let phone = dictionary?["phone"] as? String {
            setUserPhone(phone)
        }
        if let alias = dictionary?["alias"] as? [AnyHashable: Any] {
            setUserAlias(alias.toAliasDictionary())
        }
        if let attributes = dictionary?["attributes"] as? [AnyHashable: Any] {
            setUserAttributes(attributes.toAttributesDictionary())
        }
    }
}

fileprivate extension Dictionary where Key == AnyHashable, Value == Any {
    func toAliasDictionary() -> [String: String] {
        var dictionary: [String: String] = [:]
        
        forEach { key, value in
            if let key = key as? String, let value = value as? String {
                dictionary[key] = value
            }
        }
        
        return dictionary
    }

    func toAttributesDictionary() -> [String: NSObject] {
        var dictionary: [String: NSObject] = [:]
        
        forEach { key, value in
            if let key = key as? String, let value = value as? NSObject {
                dictionary[key] = value
            }
        }
        
        return dictionary
    }
}
