import 'package:airbridge_flutter_sdk/src/interface/web_interface.dart';

import 'interface/deeplink_api.dart';
import 'interface/event_api.dart';
import 'interface/state_api.dart';
import 'interface/placement_api.dart';
import 'interface/web_interface_impl.dart';

import 'data/event_option.dart';

import 'package:flutter/services.dart';

class Airbridge {

  static final _deeplinkEventChannel = EventChannel('airbridge_flutter_sdk/event/deeplink');
  static final _eventMethodChannel = MethodChannel('airbridge_flutter_sdk/method/event');
  static final _stateMethodChannel = MethodChannel('airbridge_flutter_sdk/method/state');
  static final _placementMethodChannel = MethodChannel('airbridge_flutter_sdk/method/placement');

  static final DeeplinkAPI deeplink = DeeplinkAPI(_deeplinkEventChannel);
  static final EventAPI event = EventAPI(_eventMethodChannel);
  static final StateAPI state = StateAPI(_stateMethodChannel);
  static final PlacementAPI placement = PlacementAPI(_placementMethodChannel);

  static void trackEvent(String category, [ AirbridgeEventOption? option ]) {
    event.trackEvent(category, option);
  }

  static void setDeviceAlias(String key, String value) {
    state.setDeviceAlias(key, value);
  }

  static void removeDeviceAlias(String key) {
    state.removeDeviceAlias(key);
  }

  static void clearDeviceAlias() {
    state.clearDeviceAlias();
  }

  static void registerPushToken(String token) {
    state.registerPushToken(token);
  }

  static AirbridgeWebInterface createWebInterface({
    required String webToken,
    required String Function(String arg) postCommandFunction}) {
    return AirbridgeWebInterfaceImpl(
      _eventMethodChannel,
      _stateMethodChannel,
      webToken,
      postCommandFunction);
  }
}
