import '../data/user.dart';
import '../utility/log.dart';

import 'package:flutter/services.dart';

class StateAPI {
  final MethodChannel _method;

  StateAPI(this._method);

  void setUser(User user) {
    Log.info('set user');

    _method.invokeMethod('setUser', user.toMap());
  }

  void updateUser(User user) {
    Log.info('update user');

    _method.invokeMethod('updateUser', user.toMap());
  }

  void startTracking() {
    Log.info('start tracking');

    _method.invokeMethod('startTracking');
  }

  Future<String?> get deviceUUID async {
    Log.info('get device uuid');

    return _method.invokeMethod<String?>('getDeviceUUID');
  }

  void setDeviceAlias(String key, String value) {
    Log.info('setDeviceAlias');

    Map<String, dynamic> map = {
      'key': key,
      'value': value,
    };

    _method.invokeMethod('setDeviceAlias', map);
  }

  void removeDeviceAlias(String key) {
    Log.info('removeDeviceAlias');

    Map<String, dynamic> map = {
      'key': key,
    };

    _method.invokeMethod('removeDeviceAlias', map);
  }

  void clearDeviceAlias() {
    Log.info('clearDeviceAlias');

    _method.invokeMethod('clearDeviceAlias');
  }

  void registerPushToken(String token) {
    Log.info('registerPushToken');

    Map<String, dynamic> map = {
      'token': token,
    };

    _method.invokeMethod('registerPushToken', map);
  }
}